"""Utils."""

import subprocess

from charmhelpers.contrib.network import ufw
from charmhelpers.core.hookenv import (
    config,
    log,
)


__author__ = "Felipe Reyes <felipe.reyes@canonical.com>"

FIRST = 1


def munin_format_ip(ip):
    """Get formatted ip."""
    return "^{}$".format(ip.replace(".", "\\."))


def grant_access(address):
    """Grant access to address."""
    log("granting access: {}".format(address), level="DEBUG")
    ufw.grant_access(address, port=str(config("tcp-port")), proto="tcp", index=FIRST)

    if config("udp-port") > 0:
        ufw.grant_access(
            address, port=str(config("udp-port")), proto="udp", index=FIRST
        )


def revoke_access(address):
    """Revoke access from address."""
    log("revoking access: {}".format(address), level="DEBUG")
    ufw.revoke_access(address, port=str(config("tcp-port")), proto="tcp")

    if config("udp-port") > 0:
        ufw.revoke_access(address, port=str(config("udp-port")), proto="udp")


def dpkg_info_contains(package, key, value):
    """Check if the value is contained in package metadata under the specified key.

    Return true if the value is a substring of the matching value in package metadta,
    false otherwise.
    """
    info = {}
    log("dpkg -s %s" % package, level="DEBUG")
    dpkg = subprocess.check_output(["dpkg", "-s", package]).decode("utf-8")

    for entry in dpkg.splitlines():
        entry = entry.split(":")
        if len(entry) > 1:
            info.update({entry[0].lower(): entry[1]})

    return info.get(key, -1).find(value) != -1
