"""Replication."""

from charmhelpers.contrib.hahelpers.cluster import (
    oldest_peer,
    peer_ips,
    peer_units,
)
from charmhelpers.contrib.network.ip import get_relation_ip
from charmhelpers.core.hookenv import (
    log,
    relation_id,
    relation_set,
)


REPCACHED_REPLICA_FILE = "/var/run/repcached_replica"
REPCACHED_MASTER_WAIT = 10


def store_replica(master, replica):
    """Store the replica to a file."""
    with open(REPCACHED_REPLICA_FILE, "w+") as fd:
        fd.write("%s,%s" % (master, replica))


def get_current_replica():
    """Get the current replica."""
    replica = get_repcached_replica()

    if replica:
        master, secondary = replica
        ip_addr = get_relation_ip("cluster")
        if ip_addr == master:
            replica = secondary
        elif ip_addr == secondary:
            replica = master
    else:
        peers = peer_units()

        if len(peers) > 1:
            msg = """Cannot configure replication between more than 2 units,
            all the units are going to be setup as standalone. Please
            remove the remaining units with 'juju remove-unit', and
            then set 'juju set memcached repcached=true' again"""
            log(msg, level="WARN")
        else:
            try:
                peer_unit = list(peer_ips().values())[0]

                if oldest_peer(peers):
                    master, secondary = (get_relation_ip("cluster"), peer_unit)
                    replica = secondary
                else:
                    master, secondary = (peer_unit, get_relation_ip("cluster"))
                    replica = master

                store_replica(master, secondary)

            except IndexError:
                replica = None

    log("Current replica value: %s" % replica, level="INFO")
    return replica


def get_repcached_replica():
    """Get repcached replica."""
    try:
        with open(REPCACHED_REPLICA_FILE, "r") as fd:
            return tuple(fd.read().split(","))
    except Exception:
        return None


def set_repcached_replica():
    """Set repcached replica."""
    replica = list(peer_ips().values())[0]
    log("Setting replica unit: %s" % replica)

    master = get_relation_ip("cluster")
    relation_set(
        relation_id(),
        {
            "master": master,
            "replica": replica,
        },
    )

    with open(REPCACHED_REPLICA_FILE, "w+") as fd:
        fd.write("%s,%s" % (master, replica))
    return (master, replica)
